@@ -1,4 +1,4 @@
-from typing import Tuple
+from typing import List, Optional, Tuple
 
 import torch
 from torch import Tensor
@@ -40,11 +40,12 @@ def volumetric_marching(
     rays_o: Tensor,
     rays_d: Tensor,
     aabb: Tensor,
-    scene_resolution: Tuple[int, int, int],
+    scene_resolution: List[int],
     scene_occ_binary: Tensor,
-    t_min: Tensor = None,
-    t_max: Tensor = None,
+    t_min: Optional[Tensor] = None,
+    t_max: Optional[Tensor] = None,
     render_step_size: float = 1e-3,
+    near_plane: float = 0.0,
     stratified: bool = False,
 ) -> Tuple[torch.Tensor, torch.Tensor, torch.Tensor, torch.Tensor, torch.Tensor]:
     """Volumetric marching with occupancy test.
@@ -64,6 +65,7 @@ def volumetric_marching(
         t_max: Optional. Ray far planes. Tensor with shape (n_ray,). \
             If not given it will be calculated using aabb test. Default is None.
         render_step_size: Marching step size. Default is 1e-3.
+        near_plane: Near plane of the camera. Default is 0.0.
         stratified: Whether to use stratified sampling. Default is False.
 
     Returns:
@@ -83,6 +85,8 @@ def volumetric_marching(
         raise NotImplementedError("Only support cuda inputs.")
     if t_min is None or t_max is None:
         t_min, t_max = ray_aabb_intersect(rays_o, rays_d, aabb)
+        if near_plane > 0.0:
+            t_min = torch.clamp(t_min, min=near_plane)
     assert (
         scene_occ_binary.numel()
         == scene_resolution[0] * scene_resolution[1] * scene_resolution[2]
@@ -126,7 +130,7 @@ def volumetric_rendering_steps(
     frustum_starts: Tensor,
     frustum_ends: Tensor,
     *args,
-) -> Tuple[Tensor, Tensor, Tensor]:
+) -> Tuple[Tensor, ...]:
     """Compute rendering marching steps.
 
     This function will compact the samples by terminate the marching once the \
@@ -138,8 +142,8 @@ def volumetric_rendering_steps(
 
     Args:
         packed_info: Stores infomation on which samples belong to the same ray. \
-            See volumetric_marching for details. Tensor with shape (n_rays, 2). \
-            sigmas: Densities at those samples. Tensor with shape (n_samples, 1).
+            See volumetric_marching for details. Tensor with shape (n_rays, 2).
+        sigmas: Densities at those samples. Tensor with shape (n_samples, 1).
         frustum_starts: Where the frustum-shape sample starts along a ray. Tensor with \
             shape (n_samples, 1).
         frustum_ends: Where the frustum-shape sample ends along a ray. Tensor with \
@@ -148,9 +152,9 @@ def volumetric_rendering_steps(
     Returns:
         A tuple of tensors containing
 
-            **compact_packed_info**: Compacted version of input packed_info.
-            **compact_frustum_starts**: Compacted version of input frustum_starts.
-            **compact_frustum_ends**: Compacted version of input frustum_ends.
+            - **compact_packed_info**: Compacted version of input packed_info.
+            - **compact_frustum_starts**: Compacted version of input frustum_starts.
+            - **compact_frustum_ends**: Compacted version of input frustum_ends.
 
     """
     if (
@@ -184,7 +188,7 @@ def volumetric_rendering_weights(
     sigmas: Tensor,
     frustum_starts: Tensor,
     frustum_ends: Tensor,
-) -> Tuple[Tensor, Tensor, Tensor]:
+) -> Tuple[Tensor, Tensor]:
     """Compute weights for volumetric rendering.
 
     Note: this function is only differentiable to `sigmas`.
@@ -226,8 +230,8 @@ def volumetric_rendering_weights(
 def volumetric_rendering_accumulate(
     weights: Tensor,
     ray_indices: Tensor,
-    values: Tensor = None,
-    n_rays: int = None,
+    values: Optional[Tensor] = None,
+    n_rays: Optional[int] = None,
 ) -> Tensor:
     """Accumulate volumetric values along the ray.
 
@@ -261,7 +265,7 @@ def volumetric_rendering_accumulate(
         return torch.zeros((n_rays, src.shape[-1]), device=weights.device)
 
     if n_rays is None:
-        n_rays = ray_indices.max() + 1
+        n_rays = int(ray_indices.max()) + 1
     else:
         assert n_rays > ray_indices.max()
 
